/*
   CRC32.CC
   Author: Paul Franks
   Date:   July 10, 2000

   Methods to determine CRC-32 for character strings and disk files.  
   File checksum is compatible with checksums generated by PKZIP

   Written for: Visual dBASE 7.5

   Copyright dBASE UK Ltd (2000)

   Limitations: 
     Input string or file size is limited by available memory. 

   Usage:  
     set procedure to CRC32.cc additive
     o=new CRCValue()
     ? o.GetStringCRC(cString)
     ? o.GetFileCRC(cString)
     ? o.CRC

   Returns: CRC-32 checksum of target as
            numeric variable 
            Property of CRCValue object

   Example:
      ? o.GetStringCRC("This is a test string")
      ? o.GetFileCRC("CRC32.CC")

      ?itoh(o.GetFileCRC("CRC32.CC")  
      // returns hexadecimal fingerprint
*/


Class CRCValue 
   this.f=""
   this.CRC=0
   
   Function GetFileCRC(filename)
      local sData, f
      f=new file()
      sData=""
      f.open(filename)
      do while not f.eof()
          // memory allocation bug 
         // don't try to read too much in one block
         // restricted here to 32K blocks which should be well safe
         sData=sData+(f.read(0x7fff))  
      enddo
      f.close()
   return class::GetStringCRC(sData)

   Function GetStringCRC(sData) 
      local i,nindex
      this.c=new CRCTable()
      this.c.init()   
      this.CRC=-1
      for i=1 to len(sData)
         nIndex=bitand(bitxor(asc(substr(sData,i,1)),this.CRC),0xFF)
         this.CRC=bitxor(bitzrshift(this.CRC,8),this.c[nIndex])
      endfor
      this.CRC=bitnot(this.CRC)
      release object this.c
      return this.CRC

Endclass


class CRCTable 
// no zero based arrays in dBASE
// but we can use objects
   Function Init
      local i,j
      for i=0 to 255
         this[i]=i
         for j=0 to 7
            if this[i]%2#0
               this[i]=bitxor(bitzrshift(this[i],1),0xEDB88320)
            else
               this[i]=bitzrshift(this[i],1)
            endif
         endfor
      endfor   
   return true

EndClass

