/*
   -----------------------------------------------------------------------------
   TestFTP.prg
   A sample dBL program from 
   The dBASE Book Plus by Kenneth J. Mayer. 
   Program copyright 2015. 
   Permission is granted for use as a learning tool for dBASE Plus.
   Sample from: Chapter 12
        
   This is a program designed to show how to upload a group of files
   to a file server using the Catalyst FTP ActiveX Control installed
   with dBASE Plus 9 (and later). This program is extracted from
   an application I've been using and automatically updates the files
   on a server after regenerating a series of HTML files.
   
   It requires that you fill in variables and values noted below
   (see comments). This relies on the form:
       FTPMessage.wfm
   which is also in this folder ...
   
   See also the form
      Socket FTP Example.wfm
   Contained in this folder as well.
   -----------------------------------------------------------------------------
*/
if __version__ < 9
   return
endif

//clear
// communicate with Windows better:
_app.allowYieldOnMsg := true

// set up connection to FTP Server:
oFTP = new OleAutoClient( "SocketTools.FtpClient.8" )
*****************************************************************
// fill in these variables:
oFTP.HostName := ""
oFTP.UserName := ""
oFTP.Password := ""
*****************************************************************

// attempt to connect:
nError = oFTP.connect()
if nError > 0
   // error, let the user know:
   msgbox( "Error: "+oFTP.LastErrorString+"!",;
           "FTP Connection Error", 16 )
   ? nError, oFTP.LastErrorString
   return
endif

// get to correct directory:
*****************************************************************
// You need to set the correct folder on the server -- 
// remember that any LINUX based server (most of them)
// uses forward slashes for folders, such as:
//      /public/MyPages
nError = oFTP.ChangeDirectory( "ChangeFolderName/Here" )
*****************************************************************
if nError > 0
   // error, let the user know:
   msgbox( "Error: "+oFTP.LastErrorString+"!",;
           "FTP Connection Error", 16 )
   ? nError, oFTP.LastErrorString
   oFTP.disconnect()
   return
endif

*****************************************************************
// You need to fill this out as well -- the folder the files are
// in:
cLocalPath = "C:\SomeFolderOn\YourHarddrive"
*****************************************************************

// Get array of files to upload:
aFiles = new array()
aFiles.dir( cLocalPath+"*.*" )
// number of files
nSize = aFiles.size/5

// message form:
set procedure to FTPMessage.wfm
oMsg = new FTPMessageForm()
oMsg.text               := "Uploading Files"
oMsg.ITTitle.text       := "Uploading Files ..."
oMsg.Progress1.rangeMax := nSize
oMsg.MaxText.text       := nsize
oMsg.CurrText.text      := 1
oMsg.Progress1.value    := 1
oMsg.open()

for i = 1 to nsize
    // get full path to full to upload:
    cUpload = cLocalPath+aFiles[i,1]
    // name of file on server:
    cRemote = aFiles[i,1]
    // excluding some files that don't need to be uploaded:
    if upper( right( cRemote, 3 ) ) $ "CSS,GIF,PHP"
       loop
    endif
    // msgform:
    oMsg.itMessage.text  := "Uploading "+cRemote
    oMsg.Progress1.value := i
    oMsg.CurrText.text   := i
    _app.executeMessages()
    // actually upload:
    nError = oFTP.PutFile( cUpload, cRemote )
    if nError > 0
       // error, let the user know:
       msgbox( "Error: "+oFTP.LastErrorString+"!",;
               "FTP Upload Error", 16 )
       ? nError, oFTP.LastErrorString
       oFTP.disconnect()
       return
    endif
next   

// close message form
oMsg.close()
oMsg.release()
close procedure FTPMessage.wfm
oMsg = null

// disconnect
oFTP.disconnect()

// release/null out the FTP object
release object oFTP
oFTP := null

// ****************** End of Program ************************