/*
    SHOWCURS.PRG
    04/06/1998
    Romain Strieff and Ken Mayer (minor additions for dUFLP)

    Updates: 04/06/1998 -- Gary White found that if you have
    a directory with spaces in the name, this will cause
    an error. Simple fix added to place quotes around the
    value stored for the directory in cDir.

    March 2, 1999 -- Gary made the routine 'smart' -- it 
    uses an API call to try to find the Windows\cursors 
    directory, and if it exists, it automatically loads 
    from there. If it doesn't exist it prompts you ...


    Notes:
    This program will scan the directory on your hard drive
    that contains cursor files (.CUR and .ANI); and will
    display a form that shows them when the mouse is over the
    text control with the name of the appropriate cursor.

    This uses some API calls which can be extracted from
    below ...

    (An enterprising person with some time on their hands
    might want to put together a set of routines similar
    to those in CURSOR.CC, but that use the cursors from
    files ...)

    Usage:
       do showcurs
         or
       do :dUFLP:showcurs

*/

//#define CURSDIR "c:\winnt\cursors"

// Setup:
set procedure to program(1) addi
// save current directory
cDir = '"'+set("directory")+'"'

// try to find the cursors directory
if type("GetWindowsDirectory") # "FP"
   extern CUINT GetWindowsDirectory(CSTRING, CUINT) kernel32;
      from "GetWindowsDirectoryA"
endif

CURSDIR = space(256)
GetWindowsDirectory(CURSDIR, 256)
cursdir := trim(cursdir)
cursdir := left(cursdir,len(cursdir)-1)
CURSDIR = trim(CURSDIR)+"\Cursors"

if not ( new array().dir( CURSDIR+"\*.", "D" ) > 0 )
   CURSDIR := getdir()
endif

if empty(CURSDIR)
   return
endif

// change to directory where cursors are
cd "&cursdir."

// cursor array -- loads .ANI and .CUR files
aAni=new array()
aAni.dir("*.ani")
aAni.sort()
aCur=new array()
aCur.dir("*.cur")
aCur.sort()

// create form:
local f
f=new form()
f.text = ".ANI and .CUR files available ..."
f.left=0
f.autosize=false
f.windowstate=0
f.scrollbar=1
f.width=70
f.height=20

// scan through array and load list of controls that are to be
// placed on form:
for n= 1 to aAni.size/5       // five columns by default (dir() creates five column array)
 f[n]=new cursortext(f)       // set text
 f[n].CursorFile = aAni[n,1]  // set properties for same
 f[n].hCursor = LoadCursorFromFile(aAni[n,1])
 f[n].top=n
 f[n].wrap=false
 f[n].text=aAni[n,1]
 f[n].width=f[n].GetTextExtent(aAni[n,1])
next
ns=n+1

// assign second column ...:
for n= 1 to aCur.size/5
 f[ns+n]=new cursortext(f)
 f[ns+n].CursorFile = aCur[n,1]
 f[ns+n].hCursor = LoadCursorFromFile(aCur[n,1])
 f[ns+n].top=n
 f[ns+n].left=35
 f[ns+n].wrap=false
 f[ns+n].text=aCur[n,1]
 f[ns+n].width=f[ns+n].GetTextExtent(aCur[n,1])
next

// open form
f.open()

// come back to where we started -- we're done ...
cd &cDir.

// This is the class that does all the work -- note that
// you could use this on a form if you wanted, but you 
// would need to manually set some properties ...
CLASS CURSORTEXT(FormObj) OF TEXT(FormObj) Custom
   Protect OnMouseMove,OnOpen
   this.Text="CURSORTEXT"
   this.ColorNormal = "R+/BtnFace"
   this.FontBold    = false
   this.hCursor     = 0
   this.cursorfile  = ""

   //WINAPI stuff
   if type("LoadCursorFromFile") # "FP"
      extern chandle  LoadCursorFromFile(Cstring) user32 ;
                from "LoadCursorFromFileA"
   endif
   if type("SetCursor") # "FP"
      extern CHANDLE SetCursor(CHANDLE) User32
   endif
   if type("DestroyCursor") # "FP"
      extern CLOGICAL DestroyCursor(CHANDLE) User32
   endif
   if type("SetCapture") # "FP"
      extern CHANDLE SetCapture(CHANDLE) User32
   endif
   if type("ReleaseCapture") # "FP"
      extern CVOID   ReleaseCapture() User32
   endif
    
   
   Procedure OnOpen
      if type("form.cursors") # "A"
         form.cursors=new array()
      endif      
      this.hcursor= LoadCursorFromFile(this.cursorFile)
      form.cursors.add(this.hcursor)

      if type("form.pCursorsloaded") ="U" ;
         or empty(form.pCursorsloaded)
         //paintbox which has an OnClose
         define paintbox pCursorsloaded of form ;
         property left -300
         //store array references
        //and destroy them OnClose
         form.pCursorsloaded.OnClose={;
         ;for n=1 to form.cursors.size;
         ;  DestroyCursor(form.cursors[n]);
         ; next}

      endif

   Procedure OnMouseMove(nFlags, nCol, nRow)
   
      If nCol >= 0 .AND. nCol <= this.Width .AND.;
           nRow >= 0 .AND. nRow <= this.Height 
         // If cursor is over control
         SetCursor(this.hCursor)
         SetCapture(this.hWND)
      Else  // Revert to default cursor
         ReleaseCapture()
      EndIf

ENDCLASS