// Metric: Non Metric Specific (any metric should do)
*********************************************************************
*                                                                   *
*  Author:       Marc Hamelin                                       *
*                                                                   *
*  Class Name:   ScrollImage - Version 5.3                          *
*                                                                   *
*  Instructions: The following describes how you can implement      *
*                this custom component into your own forms:         *
*                                                                   *
*                1) Basic requirements:                             *
*                   (a) ScrollImage.cc                              *
*                        - This custom component file.              *
*                   (b) dBase Plus                                  *
*                        - This class uses and requires a subform.  *
*                  ** (c) formMouseOver.cc                          *
*                          - May require formMouseOver.cc if the    *
*                            dBase Plus version is lower than 2.5.  *
*                                                                   *
*                2) This file (ScrollImage.cc) should be in your    *
*                   dUFLP directory.  The file formMouseOver.cc     *
*                   should also be in your dUFLP directory if you   *
*                   are using a dBase Plus version lower than 2.5.  *
*                                                                   *
*                3) Open this file (ScrollImage.cc) as a procedure  *
*                   file. You can do this using the SET PROCEDURE   *
*                   command. Here's an example:                     *
*                                                                   *
*                   SET PROCEDURE TO :dUFLP:ScrollImage.cc ADDITIVE *
*                                                                   *
*                4) Open or create a form in the Form Designer.     *
*                   Open the Component Palette and select the       *
*                   [Custom] tab. The ScrollImage component should  *
*                   be available. Place this component on your      *
*                   form and then resize it to suit your needs.     *
*                                                                   *
*                5) Assign an image to the dataSource of your       *
*                   newly created ScrollImage object. You can also  *
*                   specify a speedTip and an alignment setting.    *
*                   Certain alignment settings such as 0 (Stretch)  *
*                   and 2 (Center) will be converted to 3 (Keep     *
*                   Aspect Stretch) at runtime. If the alignment    *
*                   setting is set to 1 (Top Left), it will be      *
*                   converted to 4 (True Size), but its frame will  *
*                   keep its original size. Any other property will *
*                   be ignored or discarded at runtime. Also, you   *
*                   should NOT override the built-in onOpen event   *
*                   of this component, but if you really need to,   *
*                   you should include the original code in your    *
*                   own function.                                   *
*                                                                   *
*                6) Save your work and run the form. Check the      *
*                   description below to see the history of changes *
*                   for each version, but also to get the list of   *
*                   available features.                             *
*                                                                   *
*  Description:  [Version 5.3]                                      *
*                Based on a suggestion by Ken Mayer, I now check    *
*                the version of dBase Plus when the component is    *
*                opened and uses either formMouseOver.cc or the     *
*                onMouseOut event depending on the situation.       *
*                                                                   *
*                [Version 5.2]                                      *
*                With the new version of dBase Plus (2.5), I no     *
*                longer need to use the formMouseOver class by      *
*                Rich Muller. Instead, I now use the onMouseOut     *
*                event of the subform.                              *
*                                                                   *
*                [Version 5.1]                                      *
*                In this version, I corrected the centering of the  *
*                image (yet again, I know). The problem came this   *
*                time from the fact that I was comparing the width  *
*                and height of the image to determine which one     *
*                was longer. First of all, that method was not      *
*                really accurate to do this kind of verification    *
*                as the screen resolution would always skew the     *
*                result. Moreover, I soon realized that this method *
*                was completely erroneous because I was not taking  *
*                into account the size of the frame, which was also *
*                a factor in determining which dimension (the width *
*                or height) was taking all the available space of   *
*                the frame in the preview mode. Instead, I now use  *
*                a ratio based upon the frame size and the full     *
*                image size to make this verification.              *
*                                                                   *
*                IMPORTANT: I changed in the process of this update *
*                           the LongerWidth property. It is now     *
*                           called FitFrameWidth, which better      *
*                           describes its true function.            *
*                                                                   *
*                Okay, so I think I finally figured out this        *
*                centering thing. Let's hope it is truly fixed      *
*                this time (and, knock on wood, it probably is).    *
*                                                                   *
*                [Version 5]                                        *
*                In this version, I added two new functionalities   *
*                that I think are quite interesting:                *
*                                                                   *
*                1) You can now enlarge the image while in full     *
*                   view mode. To do this, you just have to hold    *
*                   the Control key and click anywhere on the image *
*                   It will then open up a small menu in which you  *
*                   can set a zoom factor ranging from 1 to 10.     *
*                                                                   *
*                2) I created a small method that allow users to    *
*                   change the dataSource easily without going      *
*                   through the hassle of coding everything up      *
*                   (which can get quite complicated in this case). *
*                   A user can now just change the dataSource       *
*                   simply by calling the AssignDataSource method.  *
*                   The parameter needs to be a character string    *
*                   containing the new dataSource. Here's an        *
*                   example:                                        *
*                                                                   *
*                   <oRef>.AssignDataSource("FILENAME C:\ABC.JPG")  *
*                                                                   *
*                Furthermore, I corrected a minor problem that had  *
*                been bugging me for quite some time. The problem   *
*                was that the centering of the image was slightly   *
*                off when resizing to its original size, and even   *
*                more so when enlarged. I thought at first that it  *
*                was caused by the fact that I was using a ratio to *
*                get the new position, but I realized with a more   *
*                careful examination that it wasn't the case. The   *
*                problem actually came from the fact that I used to *
*                put the scrollbars back on the subform and then    *
*                took its height and width values to use them in    *
*                my calculations. But since scrollbars reduce the   *
*                height and width of the subform, my calculations   *
*                were always off. It is fixed now.                  *
*                                                                   *
*                [Version 4]                                        *
*                I've decided to fit the image to the frame when    *
*                it's opened only if the alignment of the original  *
*                image object is different from 1 (Top Left)        *
*                or 4 (True Size). That way, it will be more        *
*                customizable, although the default value for the   *
*                original image object is now 3 (Keep Aspect        *
*                Stretch) instead of 1 (Top Left).                  *
*                                                                   *
*                [Version 3.3]                                      *
*                I added a small workaround to disable the          *
*                flickering of the image and scrollbars when the    *
*                image was first loaded because I was changing its  *
*                alignment setting. I also changed the way each     *
*                version description is handled.                    *
*                                                                   *
*                [Version 3.2]                                      *
*                I added a workaround for the speedTip bug, which   *
*                caused the speedTip to never appear again when the *
*                alignment of the image object was changed. I also  *
*                changed the starting alignment of the image to     *
*                make it fit within the frame so the user can see   *
*                it whole first and then choose what to examine by  *
*                zooming in.                                        *
*                                                                   *
*                [Version 3.1]                                      *
*                It's basically the very same thing as the version  *
*                3, except that it now works with the new version   *
*                of the formMouseOver component without any         *
*                workaround for the MDI verification.               *
*                                                                   *
*                [Version 3]                                        *
*                With this version, I decided to derive the         *
*                ScrollImage class from a standard Image object,    *
*                which makes it easier to implement in a form with  *
*                the Form Designer.This image is invisible and      *
*                I create the subform in the onOpen event of        *
*                the Image. It effectively becomes its Frame        *
*                property. I also copy the image's DataSource and   *
*                speedTip to the Image object within the subform    *
*                (ImageContainer). All other properties are ignored *
*                and any change after it has been opened must be    *
*                made directly at the subform level (through the    *
*                Frame property). Note, however, that the subform   *
*                is not accessible in the form's onOpen event as it *
*                is created in the onOpen event of the image        *
*                object, which is executed after the form's onOpen  *
*                event.                                             *
*                                                                   *
*                Also, I now use the formMouseOver class made by    *
*                Richard Muller (www.autotraker.com) to detect if   *
*                the mouse cursor moved out of the frame, which can *
*                now be used with any metric setting, not only      *
*                Pixels. Dragging the image, however, is much       *
*                smoother if this property is set to one of the     *
*                more finer form metrics such as Twips, Points or   *
*                Pixels.                                            *
*                                                                   *
*                [Version 2]                                        *
*                I fixed the main dragging problem using some code  *
*                by Richard Muller (www.autotraker.com). This       *
*                dragging problem was usually happening while       *
*                holding one of the mouse buttons to drag the       *
*                image around and moved the pointer outside of the  *
*                frame. If you released the mouse button while      *
*                outside of the frame's boundaries and then moved   *
*                the cursor back in, you would notice that the      *
*                image was pulled as if you were still holding the  *
*                mouse button. This was happening because the event *
*                of the image object controlling this had not been  *
*                triggered while you were outside the frame. The    *
*                metric of the form now needs to be in Pixels,      *
*                however, because otherwise this fix won't work.    *
*                                                                   *
*                [Version 1]                                        *
*                This control allows you to scroll an image that    *
*                is much larger than the space taken by the image   *
*                object on a form. There are three specific         *
*                features aside from the scrollbars themselves:     *
*                                                                   *
*                1) You can click and hold any of the two main      *
*                   mouse buttons to drag the image around within   *
*                   the frame. It is basically the standard "Grab   *
*                   and Pull" behavior found in many image viewers. *
*                                                                   *
*                2) You can doubleclick with the left mouse button  *
*                   to center the frame on the targeted position    *
*                   you just pinpointed.                            *
*                                                                   *
*                3) You can doubleclick with the right mouse button *
*                   to fit/resize the image within the boundaries   *
*                   of the frame. Doubleclicking again with either  *
*                   mouse button will cause the image to revert     *
*                   back to its normal size (True Size), and the    *
*                   frame will be roughly positioned at the same    *
*                   location where you just doubleclicked.          *
*                                                                   *
*********************************************************************

class SCROLLIMAGE(parentObj, name) of IMAGE(parentObj, name) custom
   with (this)
      onOpen = class::SCROLLIMAGE_ONOPEN
      visible = false
      alignment = 3  // Keep Aspect Stretch
   endwith


   function SCROLLIMAGE_onOpen
      // Put the original image object invisible
      this.visible = false
      // Create the subform for the frame, using the same size as 
      // the image object and the same metric as the parent form
      this.frame = new ScrollFrame(this.parent)
      this.frame.metric = this.parent.metric
      this.frame.height = this.height
      this.frame.left = this.left
      this.frame.top = this.top
      this.frame.width = this.width
      // Open the subform
      this.frame.open()
      // Assign the dataSource to the frame
      this.AssignDataSource(this.dataSource)
      // Assign the same speedTip
      this.frame.ImageContainer.speedTip = this.speedTip
      return 

   function AssignDataSource(newDataSource)
      // Put the subform invisible to remove any flickering
      this.frame.visible = false
      // Set to the full view mode to get the new image's dimensions
      if this.frame.FullSizeMode == false
         this.frame.ImageContainer.onRightDblClick(0,0,0)
      endif
      // Reset the zoom menu back to normal
      this.frame.ZoomMenu.factor.value = 1
      this.frame.ZoomMenu.factor.onChange()
      this.frame.ZoomMenu.visible = false
      // Remove the previous dataSource
      this.frame.ImageContainer.dataSource = ""
      // Set back the default property values
      this.frame.col = 0
      this.frame.row = 0
      this.frame.FullSizeMode = true
      // Create an invisible form to get the image's full dimensions
      // because it won't work (?) by setting the alignment on the
      // ImageContainer object itself
      this.dimensions = new form()
      this.dimensions.top = 0
      this.dimensions.left = 0
      this.dimensions.width = 0
      this.dimensions.height = 0
      this.dimensions.metric = this.parent.metric
      this.dimensions.visible = false
      this.dimensions.frameSize = new image(this.dimensions)
      this.dimensions.frameSize.left = 0
      this.dimensions.frameSize.top = 0
      this.dimensions.frameSize.alignment = 4
      this.dimensions.frameSize.datasource = newDataSource
      this.dimensions.open()
      // Assign the new dataSource
      this.frame.ImageContainer.dataSource = newDataSource
      // Store the image's full size dimensions
      this.frame.scroll(0,0)
      this.frame.imgFullHeight = this.dimensions.frameSize.height
      this.frame.imgFullWidth = this.dimensions.frameSize.width
      // Close and release the invisible form
      this.dimensions.close()
      this.dimensions.release()
      // Change the alignment to 3 (Keep Aspect Stretch)
      this.frame.scroll(0,0)
      this.frame.ImageContainer.alignment = 3
      this.frame.ImageContainer.height = this.frame.imgFullHeight
      this.frame.ImageContainer.width = this.frame.imgFullWidth
      // Compare the Width and Height ratios to find which of
      // these two dimensions will take all the available space
      // of the frame when in preview mode
      if (this.frame.imgFullWidth / this.frame.width) >= ;
         (this.frame.imgFullHeight / this.frame.height)
         this.frame.FitFrameWidth = true
      else
         this.frame.FitFrameWidth = false
      endif
      // Fit the image to the frame when it is opened only if the 
      // original alignment is different from 1 (Top Left) or
      // 4 (True Size)
      if this.alignment == 0 OR this.alignment == 2 OR ;
         this.alignment == 3
         this.frame.ImageContainer.onRightDblClick(0,0,0)
      endif
      // Remove the dataSource of the image object (if any) so that
      // the image is not opened twice
      this.dataSource = ""
      // Set the subform back to visible
      this.frame.visible = true      
      return

endclass


class SCROLLFRAME(form) of SUBFORM(form)
   with (this)
      onOpen = class::FORM_ONOPEN
      onMouseMove = class::FORM_ONMOUSEMOVE
      canClose = class::FORM_CANCLOSE
      height = 0
      left = 0
      top = 0
      width = 0
      clientEdge = true
      smallTitle = true
      sysMenu = false
      escExit = false
      minimize = false
      maximize = false
      moveable = false
      sizeable = false
      scrollBar = 2  // Auto
      visible = false
      text = ""
   endwith

   this.IMAGECONTAINER = new IMAGE(this)
   with (this.IMAGECONTAINER)
      onMouseMove = class::IMAGECONTAINER_ONMOUSEMOVE
      onLeftMouseDown = class::IMAGECONTAINER_ONMOUSEBUTTONDOWN
      onLeftMouseUp = class::IMAGECONTAINER_ONMOUSEBUTTONUP
      onRightMouseDown = class::IMAGECONTAINER_ONMOUSEBUTTONDOWN
      onRightMouseUp = class::IMAGECONTAINER_ONMOUSEBUTTONUP
      onLeftDblClick = class::IMAGECONTAINER_ONLEFTDBLCLICK
      onRightDblClick = class::IMAGECONTAINER_ONRIGHTDBLCLICK
      height = 0
      left = 0
      top = 0
      width = 0
      alignment = 3  // Keep Aspect Stretch
      borderStyle = 3  // None
      dataSource = ""
   endwith


   function IMAGECONTAINER_onMouseMove(flags, col, row)
      // Scroll the frame if the one of the mouse button is 
      // pressed (dragging mode)
      if this.MousePointer == 13
         form.scroll(form.ScrollVOffset - (row - form.row), ;
            form.ScrollHOffset - (col - form.col))
      endif
      return

   function IMAGECONTAINER_onMouseButtonDown(flags, col, row)
      // Change the pointer when the button is pressed and keep
      // the original position coordinates
      this.MousePointer = 13
      form.col = col
      form.row = row
      return

   function IMAGECONTAINER_onMouseButtonUp(flags, col, row)
      // Reset the pointer and the coordinates when the button
      // is released
      this.MousePointer = 0
      form.col = 0
      form.row = 0
      // Open the zoom factor menu when the user clicks
      // on the image and the Control key is pressed
      if bitset(flags,3) = true
         form.ZoomMenu.top = 0
         form.ZoomMenu.left = 0
         form.ZoomMenu.visible = true
      endif
      return

   function IMAGECONTAINER_onLeftDblClick(flags, col, row)
      // Center on the position where the user doubleclicked
      if form.FullSizeMode == true
         // Scroll to the position if in full size mode and the
         // Control key is not pressed
         local nTop, nLeft
         nTop = form.ZoomMenu.top
         nLeft = form.ZoomMenu.left
         form.scroll(row - (form.height / 2), col - (form.width / 2))
         form.ZoomMenu.top = nTop
         form.ZoomMenu.left = nLeft
         release nTop
         release nLeft
      else
         // Call the Right Doubleclick function if the image fits
         // the frame
         this.onRightDblClick(flags, col, row)
      endif
      return

   function IMAGECONTAINER_onRightDblClick(flags, col, row)
      if form.FullSizeMode == true
         // Scroll back at the beginning and resize the image to
         // fit within the frame
         local nTop, nLeft
         nTop = form.ZoomMenu.top
         nLeft = form.ZoomMenu.left
         form.scroll(0,0)
         form.scrollBar = 0
         this.height = form.height
         this.width = form.width
         form.ZoomMenu.top = nTop
         form.ZoomMenu.left = nLeft
         release nTop
         release nLeft
         // Change the display mode
         form.FullSizeMode = false
      else
         // Reset the image to its full size and scroll roughly
         // to the same place where the user doubleclicked
         local fullWidth, fullHeight
         local fullFrameWidth, fullFrameHeight
         fullWidth = form.imgFullWidth * form.ZoomMenu.factor.value
         fullHeight = form.imgFullHeight * form.ZoomMenu.factor.value
         fullFrameWidth = this.width
         fullFrameHeight = this.height
         this.height = fullHeight
         this.width = fullWidth
         form.scrollBar = 2
         local nTop, nLeft
         nTop = form.ZoomMenu.top
         nLeft = form.ZoomMenu.Left
         // Scroll back to the equivalent position
         if form.FitFrameWidth == true
            form.scroll(((row - ((fullFrameHeight - ;
            ((fullFrameWidth * fullHeight) / fullWidth)) / 2)) * ;
            (fullWidth / fullFrameWidth)) - (form.height / 2), ;
            (col * (fullWidth / fullFrameWidth)) - (form.width / 2))
         else
            form.scroll((row * (fullHeight / fullFrameHeight)) - ;
            (form.height / 2), ((col - ((fullFrameWidth - ;
            ((fullFrameHeight * fullWidth) / fullHeight)) / 2)) * ;
            (fullHeight / fullFrameHeight)) - (form.width / 2))
         endif
         form.ZoomMenu.top = nTop
         form.ZoomMenu.left = nLeft
         release nTop
         release nLeft
         release fullWidth
         release fullHeight
         release fullFrameWidth
         release fullFrameHeight
         // Change the display mode
         form.FullSizeMode = true
      endif
      // This is to fix a bug causing the speedTip to never appear 
      // again when the alignment of the image has been changed
      this.speedTip = this.speedTip
      return

   function FORM_onMouseMove(flags, col, row)
      // Reset the pointer and the coordinates when the cursor moves
      // out of the image object
      form.ImageContainer.MousePointer = 0
      form.col = 0
      form.row = 0
      return

   function FORM_MouseOut(flags, col, row)
      // Reset the pointer and the coordinates when the cursor moves
      // out of the frame (subform)
      form.ImageContainer.MousePointer = 0
      form.col = 0
      form.row = 0
      return

   function FORM_onOpen
      // Verify the dBase Plus version and uses formMouseOver.cc if
      // it is lower than 2.5 because the onMouseOut event does not
      // exist in previous versions of dBase Plus
      if __version__ >= 2.5
         form.onMouseOut = class::FORM_MouseOut
      else
        **************************************************
         set procedure to :dUFLP:formMouseOver.cc additive
        **************************************************
         form.formMouseOverEvents = new FORMMOUSEOVER(form)
         form.formMouseOverEvents.left = 0
         form.formMouseOverEvents.top = 0
      endif
      // Create the zoom factor menu
      form.ZoomMenu = new ZoomFactor(form)
      form.ZoomMenu.open()
      // Set the default property values
      form.col = 0
      form.row = 0
      form.FullSizeMode = true
      form.FitFrameWidth = false
      form.imgFullHeight = 0
      form.imgFullWidth = 0
      return

   function FORM_canClose
      // Remove the canClose event of the zoom menu so that
      // it can be closed
      form.ZoomMenu.canClose = null
      return true

endclass


class ZOOMFACTOR(form) of SUBFORM(form)
   with (this)
      canClose = class::FORM_CANCLOSE
      height = 1.35
      left = 0
      top = 0
      width = 20
      text = "Zoom Factor"
      smallTitle = true
      sizeable = false
      escExit = false
      maximize = false
      minimize = false
      visible = false
   endwith

   this.FACTOR = new SLIDER(this)
   with (this.FACTOR)
      onChange = class::FACTOR_ONCHANGE
      speedTip = "Zoom (x1)"
      height = 1.5
      left = 0
      top = 0
      width = 20
      value = 1
      rangeMax = 10
      rangeMin = 1
      vertical = false
      enableSelection = false
      startSelection = -1
      endSelection = -1
   endwith


   function FACTOR_onChange
      // Change the description
      this.speedTip = "Zoom (x" + rtrim(ltrim(str(this.value))) + ")"
      // Resize the image if in full size mode
      if form.parent.FullSizeMode == true
         // Check if the zoom menu will be out of the image and
         // reposition the frame if it's the case
         if form.parent.ScrollVOffset >= ;
            (form.parent.imgFullHeight * this.value) OR ;
            form.parent.ScrollHOffset >= ;
            (form.parent.imgFullWidth * this.value)
            local nTop, nLeft
            nTop = form.top
            nLeft = form.Left
            form.parent.scroll((form.parent.imgFullHeight * ;
               this.value) - form.parent.height, ;
               (form.parent.imgFullWidth * this.value) - ;
               form.parent.width)
            form.top = nTop
            form.left = nLeft
            release nTop
            release nLeft
         endif
         // Change the image size
         form.parent.imagecontainer.height = ;
            form.parent.imgFullHeight * this.value
         form.parent.imagecontainer.width = ;
            form.parent.imgFullWidth * this.value
         form.parent.scrollBar = 2
      endif
      return

   function FORM_canClose
      // Set the form invisible
      form.visible = false
      return false

endclass

