PROCEDURE CreTable7
/*
   ----------------------------------------------------------------
   Programmer......: Romain Strieff 
   Date............: 02/20/1996
   History.........: Version 1.0  09/01/1995
                     Version 1.0a 02/10/1996 (code capitaliz. 
                                             changed)
                     Version 1.1  02/20/1996 (can create any field)
                     Version 2.0  05/03/1999 (updated for Visual 
                                              dBASE 7)

   Modified by.....: Gary White (gary@apptools.com) 05/05/1999

   Notes...........: This procedure called with the name of a DBF 
                     or wild-card (with extension) creates the 
                     needed SQL command CREATE TABLE code to 
                     recreate it/them from scratch. It also adds 
                     code for recreating all existing index tags. 
                     It will create a CRETABLE.TXT file to 
                     hold the code. Incorporate this code in your
                     program to create new DBFS on installation or 
                     use for posting problems in CIS forum or 
                     DB2K news group messages to allow the 
                     reader recreating the same structure you 
                     are using.

   Written for.....: Visual dBASE 7.x

   Called by.......: command window (normally)

   Usage...........: CreTable7("<cDBF.dbf>", bCopyToClip)

   Example.........: CreTable7("ORDERS.DBF", true)
                     CreTable7("*.DBF")

   Returns.........: None

   Creates.........: CRETABLE.TXT, containing all the data needed to
                     create a copy of the dbfS with the index tags.
                     Will overwrite and existing CRETABLE.TXT file
                     without warning.

   Parameters......: cDBF = Character string containing the name of 
                     a TABLE file or Wildcard

                     bCopyToClip = Logical value determines if the text
                     is copied to the Windows clipboard. Default value 
                     is false. The copy routine is a little slow, but it
                     is still faster than opening the text file and 
                     copying manually.

   Caveats:

      There is possible ambiguity in identifying a primary key.
      Primary keys are identified only by the index expression.
      This means that, if you have another index with the same
      index expression as the primary key, the wrong one may be
      chosen and generated as primary.

      You may, for example, have a primary key based upon one 
      expression and have another index on the same expression
      where one is in ascending order and the other is in descending
      order. There is no way to determine which one is the primary
      key. The first one encountered is the one that will be used.

   ----------------------------------------------------------------
*/

parameters cDBF, bCopyToClip

if pcount()=0 .or. .not. ".DBF" = right(upper(cDBF),4)
   ?? chr(7)
   msgbox('You must call this Function with the name of' + chr(13) + ;
    + 'an existing Table ( "MYDBF.DBF" ) or with a' + chr(13) ;
    + 'Wildcard expression ( "*.DBF" ) !', 'Error!', 16)
   RETURN
endif // pcount()=0

private all except _*

if empty( bCopyToClip )
	bCopyToClip = false
endif

// build an array of the file(s) asked
aFiles=new array()
afiles.dir(cDBF)
if aFiles.size == 0     // if none found
   msgbox("No files found to process!","Error!",16)   // tell the user
   release object aFiles
   return   // and quit
endif

sAlias=alias()

cOutfile="CRETABLE.TXT"

sTalk=set("TALK")
set talk off
sSafe=set("SAFETY")
set safety off
sLdcheck=set("LDCHECK")
set ldcheck off

set alternate to (coutfile)
set alternate on

// main loop through all the files
for nCounter=1 to aFiles.size/5
   cDBF=aFiles[nCounter,1]
   cAlias=left(cDBF,at('.',cDBF)-1)

   ? "*---------Code for creating "+cDBF

   aNames = new array()    // field names
   aTypes = new array()    // field types
   aSize = new array()     // field lengths
   aDec = new array()      // field number of decimals

   t=new tabledef()
   t.tablename = cDBF
   try
      t.load()
   catch ( exception e )
      msgbox(e.message, "Error: " + e.code, 16 )
      exit
   endtry
   nFields=t.fields.size
   lConverted = false
   for i = 1 to t.fields.size
      // don't add _DBASELOCK field to arrays
      if t.fields[i].fieldName == "_DBASELOCK"
         nFields --
         lConverted = true
         // length of _DBASELOCK field is not surfaced
         // in the TableDef object, so we need to open
         // the file with XDML to get it.
         use (cDBF) noupdate in select()
         for n=1 to fldcount()
            if field(n)=="_DBASELOCK"
               nConverted = flength(n)
               exit
            endif // if field(n)=="_DBASELOCK"
         endfor   // for n=1 to fldcount()
         use
      else
         aNames.add( t.fields[i].fieldName )
         aTypes.add( t.fields[i].type )
         aSize.add( t.fields[i].length )
         aDec.add( t.fields[i].decimalLength )
      endif // if aNames[i] == "_DBASELOCK"
   endfor   // for i = 1 to t.fields.size
   
   ? 'if .not. file ( "'+upper(cDBF)+'" )'
   cCmd = '   create table "'+upper(cDBF)+ '" ( ;' + chr(13)
   bAltering = false

   // LocalSql cannot create a FLOAT so we need to keep 
   // track of FLOAT fields that might be in the table.
   aFloat=new array()

   // loop through fields of the dbf
   for n=1 to nFields
      cCmd += "      "+iif(bAltering,'add ','')+ cAlias+".'"+ aNames[n]+"'"
      
      // now determine the parameters the SQL command needs
      do case
         case aTypes[n]=="CHARACTER"
            cType= "CHAR"
            cType+="("+ltrim(str(aSize[n]))+")"
         case aTypes[n]=="NUMERIC"
            cType= "NUMERIC"
            cType=cType+"("+ltrim(str(aSize[n]))+ ",";
             +ltrim(str(aDec[n])) +")"
         case aTypes[n]=="DATE"
            cType ="DATE"
         case aTypes[n]=="FLOAT"  // LocalSql doesn't support creation
            cType ="NUMERIC"     // of dBASE FLOAT fields, so we create
            cType=cType+"("+ltrim(str(aSize[n]))+ ",";   // a NUMERIC
             +ltrim(str(aDec[n])) +")"    // we'll change it later
             aFloat.add(n)                // remember if for later
         case aTypes[n]=="BINARY"
            cType ="BLOB(10,2)"
         case aTypes[n]=="OLE"
            cType ="BLOB(10,4)"
         case aTypes[n]=="MEMO"
            cType ="BLOB(10,1)"
         case aTypes[n]=="LOGICAL"
            cType ="BOOLEAN"
         case aTypes[n]=="LONG"
            cType = "INTEGER"
         case aTypes[n]=="DATETIME"
            cType = "TIMESTAMP"
         case aTypes[n]=="DOUBLE"
            cType = "FLOAT"
            cType=cType+"("+ltrim(str(aSize[n])) + ")"
         case aTypes[n]=="AUTOINC"
            cType = "AUTOINC"
      endcase
      // print type of field
      cCmd += " " + cType
      // check if this is last field or not
      if n=nfields         // no primary key
         ? cCmd
      	if not bAltering
         	?? ")"
         endif
      else
         cCmd += ",;" + chr(13)
      endif // n=nfields

      if len( cCmd ) > 1024
      	cCmd := stuff( cCmd, len(cCmd)-2, 2, iif(bAltering,"",")"))
      	? cCmd
         bAltering = true
		   cCmd = '   alter table "'+ upper(cDBF) + '" ;' + chr(13)
      endif
      
   next // n=1 to nFields

   // if we have any FLOAT fields, we need to change the types
   // to match using low-level file access
   if aFloat.size > 0

      ? "   f = new file()"
      ? "   f.open('" + cDBF + "', 'RW' )"
      for i = 1 to aFloat.size
         ? "   f.seek( 52 + 48 *",ltrim('' + aFloat[i]),", 0 )"
         ? "   f.write( 'F' )"
      endfor
      ? "   f.close()"
      ? "   release object f"

   endif
      
   ? "endif"
   ?

   // now check the index tags
   // -- only perform routine if an index tag exists
   nPrimary = 0
   for i = 1 to t.indexes.size
   
      if t.indexes[i].expression == t.primaryKey
         nPrimary ++
      endif
      if t.indexes[i].expression == t.primaryKey and nPrimary < 2
         ? "use",t.tableName,"exclusive in select()"
         // if table is converted, handle that here since
         // we have the table open exclusive anyway
         if lconverted
            ? "convert to",ltrim( '' + nConverted )
            lConverted := false
         endif
         // modified June 4, 2001, to include ":" delimiter
         // around field names so that names with spaces
         // are not a problem
         ? "index on",t.indexes[i].expression,;
            "tag :",t.indexes[i].indexName+":"
         if t.indexes[i].descending
            ?? " descending"
         endif
         ?? " primary"
         ? "use"
         ?
      else
         ? "oIndex = new dbfIndex()"
         ? "with (oIndex)"
         ? "   descending =",t.indexes[i].descending
         if not empty( t.indexes[i].expression )
            ? "   expression = '" + t.indexes[i].expression + "'"
         endif
         if not empty( t.indexes[i].forExpression )
            ? "   forExpression = '" + t.indexes[i].forExpression + "'"
         endif
         ? "   indexName = '" + t.indexes[i].indexName + "'"
         ? "   unique =",t.indexes[i].unique
         ? "endwith"
         ? "_app.databases[1].createIndex( '" + cDBF + "', oIndex )"
         ? "release object oIndex"
         ?
      endif
   endfor

   if lconverted
      ? "use '" + upper(cDBF) + "' exclusive in select()"
      ? "convert to",ltrim( '' + nConverted )
      ? "use"
   endif
      
   use    // close database
   ? "*---------End "+cDBF
   ?
   ?
   release object aNames
   release object aTypes
   release object aSize
   release object aDec
   release object t
   release object aFloat

next // nCounter=1 to aFiles.size/5

cMsg = "The created code is in CRETABLE.TXT"
nBoxType = 64
cBoxTitle = "Done!"

set alternate off
set alternate to

if nPrimary > 1
   nBoxType = 48
   cBoxTitle = "Alert!"
   cMsg += chr(13) + chr(13) + "Note that the table included "+nPrimary+;
      " indexes with the same expression" + chr(13) + ;
      "as the primary key Check the Generated Source code carefully" + chr(13) + ;
      "to ensure the correct index was specified as a primary key!"
//   modi comm (cOutFile)
endif

if bCopyToClip			// copies the text to the clipboard
   f = new file()
   f.open('cretable.txt', 'R')
   c = f.read( 65535 )
   f.close()
   release object f

   frm = new form()
   frm.top=-300
   frm.e=new entryfield(frm)
   frm.open()
   frm.e.value=c
   frm.e.setfocus()
   frm.e.copy()
   frm.close()
   frm.release()
   release object frm
   cMsg += chr(13) + chr(13) + "Text has been copied to the Windows clipboard."
endif

if .not. empty(sAlias)
   select (salias)
endif // .not. empty(sAlias)

set talk &stalk.
set safety &sSafe.
set ldcheck &sldcheck.

release object aFiles

?? chr(7)
msgbox( cMsg, cBoxTitle, nBoxType )
RETURN

