/*
   Lookup Example.prg
   A sample dBL program from 
   The dBASE Book
   by Kenneth J. Mayer. 
   Program copyright 2012. 
   Permission granted for use as a learning tool for dBASE Plus.

   Sample from: Chapter 8 
   This code is designed to show the basics of using a lookup
   in another table, with the field object's lookupSQL property,
   and to show how to see the original value stored in the main
   table. This also shows how to use lookupRowset.
*/

clear

// If table does not exist, run routine
// for this chapter that creates the tables ...
if not file("AddressBook.dbf")
   do MakeChapter8Tables
endif


// set up initial query:
q = new query()
q.sql       := "select * from AddressBook"
q.active    := true
q.rowset.indexName := "Name"

// establish the lookup:
q.rowset.fields["StateProv"].lookupSQL := "select * from StateProv order by Code"

? "------------------------------------------------------------------------------"
? "lookupSQL examples:"
?
? "Show the value from the lookup table:"
?

// loop through table and show "value" in the field:
q.rowset.first()
do while not q.rowset.endOfSet
   // display the value:
   ? q.rowset.fields["StateProv"].value
   // navigate to next row:
   q.rowset.next()
enddo

? // blank line
? "Show the values contained in the field, as well as the value from the lookup table:"
?

// loop through table and show "value" in the field:
q.rowset.first()
do while not q.rowset.endOfSet
   // display the value:
   ? q.rowset.fields["StateProv"].value.rightTrim()+" ("+;
     q.rowset.fields["StateProv"].lookupRowset.fields[1].value.rightTrim()+")"
   // navigate to next row:
   q.rowset.next()
enddo

// Example of lookupRowset instead of lookupSQL:

?
? "------------------------------------------------------------------------------"
? "lookupRowset example:"
?

// first, disconnect the lookupSQL:
q.rowset.fields["StateProv"].lookupSQL := null

// Establish the code for the lookup query:
qState = new query()
qState.sql := "select Code, StateProv from StateProv"
qState.active := true

// establish the lookupRowset property:
q.rowset.fields["StateProv"].lookupRowset := qState.rowset

// loop through table and show "value" in the field:
q.rowset.first()
do while not q.rowset.endOfSet
   // display the value:
   ? q.rowset.fields["StateProv"].value.rightTrim()+" ("+;
     q.rowset.fields["StateProv"].lookupRowset.fields[1].value.rightTrim()+")"
   // navigate to next row:
   q.rowset.next()
enddo


// cleanup
q.active := false
release object q
q := null

qState.active := false
release object qState
qState := null

