/*
   MyReportControls.cc
   A sample dBL custom class file from 
   The dBASE Reports Book
   by Kenneth J. Mayer. 
   Program copyright 2007. 
   Permission granted for use as a learning tool for dBASE Plus.

   Classes used in reports throughout the tutorial aspects
   of the book ...
*/
class MAINTITLETEXT(parentObj) of TEXT(parentObj) custom
   with (this)
      width           = 2.81
      height          = 0.33
      prefixEnable    = false
      alignHorizontal = 1	// Center
      fontSize        = 20
      fontItalic      = true
      text            = "My Company Name"
   endwith
endclass

class SECONDARYTITLETEXT(parentObj) of TEXT(parentObj) custom
   with (this)
      width           = 4.11
      height          = 0.25
      prefixEnable    = false
      alignHorizontal = 1	// Center
      fontSize        = 16
      text            = "Purpose of Report"
   endwith
endclass

class DATETEXT(parentObj) of TEXT(parentObj) custom
   with (this)
      width           = 2.00
      height          = 0.21
      prefixEnable    = false
      alignHorizontal = 1	// Center
      fontSize        = 12
      text            = {||cmonth(date())+" "+day(date())+", "+year(date())}
   endwith

endclass

class DATATEXT(parentObj) of TEXT(parentObj) custom
   with( this )
      fontName = "Arial"
      fontSize = 10
      text     = "DataText"
   endwith
endclass

class DATAHEADINGTEXT(parentObj) of TEXT(parentObj) custom
   with( this )
      text     = "HeadingText"
      fontName = "Arial"
      fontSize = 10
      fontBold = true
   endwith
endclass

class DATACOLHEADINGTEXT(parentObj) of DATAHEADINGTEXT(parentObj) custom
   with( this )
      text            = "ColumnHeadingText"
      suppressIfBlank = true
      canRender       = {||this.parent.firstOnFrame}
   endwith
endclass

class PageNum( parentObj ) of DataText( parentObj ) custom
   // Class that outputs "Page n" where "n" is the current
   // page number -- the value of the report's reportPage
   // property:
   with( this )
      fontSize = 8
      text = class::PageValue
   endwith

   function PageValue
      local cPage
      cPage = "Page "
      if type( "this.form.reportPage" ) == "N"
         // add page number to end of text
         cPage += this.form.reportPage
      else
         // if this is not used on a report,
         // return word "ERROR"
         cPage := "ERROR"
      endif
   return( cPage )
endclass

class PageNumOffset( parentObj ) of DataText( parentObj ) custom
   // Page Number Offset class, this offsets the report's
   // reportPage property by some number, so that the first
   // page of the report will be form.reportPage+offsetvalue
   // When using this class, place it on a report, and then
   // in the text control's onOpen event, enter: 
   //       {; this.pageOffset := 2 }
   // where "2" is the amount you wish to add to the 
   // reportPage value to offset the page number.
   with( this )
      fontSize = 8
      text = class::PageValue
   endwith
   this.pageOffset = 0 // default to zero

   function PageValue
      local nPage
      nPage = 0
      if type( "this.form.reportPage" ) == "N"
         // add page number to end of text
         nPage = this.form.reportPage + this.pageOffset
      else
         // if this is not used on a report,
         // return word zero
         nPage := 0
      endif
   return( nPage )
endclass
